package com.displaynone.acss.components.auth.models.user

import android.content.Context
import com.displaynone.acss.components.auth.internal_utils.AuthTokenManager
import com.displaynone.acss.components.auth.models.user.repository.UserRepository
import com.displaynone.acss.components.auth.models.user.repository.dto.LastVisitsDto
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import com.displaynone.acss.components.auth.models.user.repository.dto.VisitDto


class UserServiceST(
    private val tokenManager: AuthTokenManager,
) {
    private val userRepository: UserRepository = UserRepository()

    companion object {
        private var instance: UserServiceST? = null

        fun createInstance(context: Context) {
            if (instance == null) {
                val tokenManager =
                    AuthTokenManager(
                        context
                    )
                instance = UserServiceST(tokenManager)
            }
        }

        fun getInstance(): UserServiceST {
            return instance ?: throw RuntimeException("null instance")
        }
    }
    suspend fun login(login: String, password:String): Result<Unit>{
        return runCatching {
            userRepository.login(login = login, password).getOrThrow().let { data ->
                tokenManager.saveTokens(data)
            }
        }
    }
    suspend fun getMyLastVisits(pageNum: Int,
                                pageSize: Int): Result<List<VisitDto>> {
        if (!tokenManager.hasTokens()) {
            throw RuntimeException("access token is null")
        }

        return userRepository.getMyLastVisits(
            pageNum = pageNum,
            pageSize = pageSize,
            token = tokenManager.authTokenPair!!.accessToken
        ).map { pagingDto -> pagingDto.content
        }
    }
    suspend fun getLastVisitsByLogin(pageNum: Int,
                                     pageSize: Int,
                                     login: String): Result<List<VisitDto>> {
        if (!tokenManager.hasTokens()) {
            throw RuntimeException("access token is null")
        }
        return userRepository.getLastVisitsByLogin(pageNum, pageSize, tokenManager.authTokenPair!!.accessToken, login).map { pagingDto -> pagingDto.content }
    }
    fun logout(){
        tokenManager.clear()
    }
    suspend fun getInfo(): Result<UserDTO>{
        if (!tokenManager.hasTokens()) {
            throw RuntimeException("access token is null")
        }
        return userRepository.getInfo(tokenManager.authTokenPair!!.accessToken)
    }
    suspend fun getInfoByLogin(login: String): Result<UserDTO>{
        if (!tokenManager.hasTokens()) {
            throw RuntimeException("access token is null")
        }
        return userRepository.getInfoByLogin(tokenManager.authTokenPair!!.accessToken, login)
    }
    suspend fun openDoor(code: String): Result<Int> {
        return userRepository.openDoor(tokenManager.authTokenPair!!.accessToken, code = code)
    }
}