package com.displaynone.acss.ui.profile

import android.util.Log
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.cachedIn
import androidx.paging.log
import com.displaynone.acss.components.auth.models.user.UserServiceST
import com.displaynone.acss.components.auth.models.user.repository.VisitListPagingSource
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import kotlinx.coroutines.channels.BufferOverflow
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.receiveAsFlow
import kotlinx.coroutines.launch

class ProfileViewModel(): ViewModel() {
    private val _action = Channel<Action>(
        capacity = Channel.BUFFERED,
        onBufferOverflow = BufferOverflow.DROP_OLDEST,
    )
    private var login: String = "" // FIXME()
    fun setLogin(login1: String){
        login = login1
    }
    val action = _action.receiveAsFlow()
    val _state = MutableStateFlow<State>(State.Loading)
    val state = _state.asStateFlow()

    val visitListState = Pager(
        config = PagingConfig(pageSize = 20,
            enablePlaceholders = false,
            maxSize = 100
        )
    ) {
        VisitListPagingSource(UserServiceST.getInstance()::getMyLastVisits)
    }.flow
        .cachedIn(viewModelScope)

    val visitListStateFromLogin = Pager(
        config = PagingConfig(pageSize = 20, enablePlaceholders = false, maxSize = 100)
    ) {
        VisitListPagingSource { pageNum, pageSize ->
            UserServiceST.getInstance().getLastVisitsByLogin(pageNum,pageSize, login)
        }
    }.flow.cachedIn(viewModelScope)

    fun logout(){
        UserServiceST.getInstance().logout()
    }
    fun getInfo(){
        viewModelScope.launch {
            UserServiceST.getInstance().getInfo().fold(
                onSuccess = { data ->
                    _state.emit(State.Show(data))
                },
                onFailure = { error ->
                    error.message?.let { error(it) }
                    Log.e("ProfileViewModel", error.message.toString())
                }
            )
        }
    }
    fun openAuth(){
        viewModelScope.launch {
            _action.send(Action.GoToAuth)
        }
    }
    fun openScan(){
        viewModelScope.launch {
            _action.send(Action.GoToScan)
        }
    }

    sealed interface State {
        data object Loading : State
        data class Show(
            val item: UserDTO
        ) : State
    }
    sealed interface Action {
        data object GoToAuth: Action
        data object GoToScan: Action
    }
}